@@ -63,8 +63,7 @@ HTS_MODEL_C_START;
 #include "HTS_hidden.h"
 
 /* HTS_dp_match: recursive matching */
-static HTS_Boolean HTS_dp_match(const char *string, const char *pattern,
-                                const int pos, const int max)
+static HTS_Boolean HTS_dp_match(const char *string, const char *pattern, const int pos, const int max)
 {
    if (pos > max)
       return FALSE;
@@ -105,8 +104,7 @@ static HTS_Boolean HTS_pattern_match(const char *string, const char *pattern)
          max++;
       }
    }
-   if (nstar == 2 && nquestion == 0 && pattern[0] == '*'
-       && pattern[i - 1] == '*') {
+   if (nstar == 2 && nquestion == 0 && pattern[0] == '*' && pattern[i - 1] == '*') {
       /* only string matching is required */
       buff_length = i - 2;
       for (i = 0, j = 1; i < buff_length; i++, j++)
@@ -158,22 +156,35 @@ static int HTS_get_state_num(char *string)
    return atoi(string);
 }
 
+static void HTS_Question_clear(HTS_Question * question);
+
 /* HTS_Question_load: Load questions from file */
-static void HTS_Question_load(HTS_Question * question, FILE * fp)
+static HTS_Boolean HTS_Question_load(HTS_Question * question, HTS_File * fp)
 {
    char buff[HTS_MAXBUFLEN];
    HTS_Pattern *pattern, *last_pattern;
 
+   if (question == NULL || fp == NULL)
+      return FALSE;
+
    /* get question name */
-   HTS_get_pattern_token(fp, buff);
+   if (HTS_get_pattern_token(fp, buff) == FALSE)
+      return FALSE;
    question->string = HTS_strdup(buff);
    question->head = NULL;
    /* get pattern list */
-   HTS_get_pattern_token(fp, buff);
+   if (HTS_get_pattern_token(fp, buff) == FALSE) {
+      free(question->string);
+      question->string = NULL;
+      return FALSE;
+   }
    last_pattern = NULL;
    if (strcmp(buff, "{") == 0) {
       while (1) {
-         HTS_get_pattern_token(fp, buff);
+         if (HTS_get_pattern_token(fp, buff) == FALSE) {
+            HTS_Question_clear(question);
+            return FALSE;
+         }
          pattern = (HTS_Pattern *) HTS_calloc(1, sizeof(HTS_Pattern));
          if (question->head)
             last_pattern->next = pattern;
@@ -181,17 +192,20 @@ static void HTS_Question_load(HTS_Question * question, FILE * fp)
             question->head = pattern;
          pattern->string = HTS_strdup(buff);
          pattern->next = NULL;
-         HTS_get_pattern_token(fp, buff);
+         if (HTS_get_pattern_token(fp, buff) == FALSE) {
+            HTS_Question_clear(question);
+            return FALSE;
+         }
          if (!strcmp(buff, "}"))
             break;
          last_pattern = pattern;
       }
    }
+   return TRUE;
 }
 
 /* HTS_Question_match: check given string match given question */
-static HTS_Boolean HTS_Question_match(const HTS_Question * question,
-                                      const char *string)
+static HTS_Boolean HTS_Question_match(const HTS_Question * question, const char *string)
 {
    HTS_Pattern *pattern;
 
@@ -203,8 +217,7 @@ static HTS_Boolean HTS_Question_match(const HTS_Question * question,
 }
 
 /* HTS_Question_find_question: find question from question list */
-static HTS_Question *HTS_Question_find_question(HTS_Question * question,
-                                                const char *buff)
+static HTS_Question *HTS_Question_find_question(HTS_Question * question, const char *buff)
 {
 
    for (; question; question = question->next)
@@ -285,27 +298,51 @@ static void HTS_Tree_parse_pattern(HTS_Tree * tree, char *string)
    }
 }
 
+static void HTS_Tree_clear(HTS_Tree * tree);
+
 /* HTS_Tree_load: Load trees */
-static void HTS_Tree_load(HTS_Tree * tree, FILE * fp, HTS_Question * question)
+static HTS_Boolean HTS_Tree_load(HTS_Tree * tree, HTS_File * fp, HTS_Question * question)
 {
    char buff[HTS_MAXBUFLEN];
    HTS_Node *node, *last_node;
 
-   HTS_get_pattern_token(fp, buff);
+   if (tree == NULL || fp == NULL)
+      return FALSE;
+
+   if (HTS_get_pattern_token(fp, buff) == FALSE) {
+      HTS_Tree_clear(tree);
+      return FALSE;
+   }
    node = (HTS_Node *) HTS_calloc(1, sizeof(HTS_Node));
    node->index = 0;
    tree->root = last_node = node;
 
    if (strcmp(buff, "{") == 0) {
-      while (HTS_get_pattern_token(fp, buff), strcmp(buff, "}") != 0) {
+      while (HTS_get_pattern_token(fp, buff) == TRUE && strcmp(buff, "}") != 0) {
          node = HTS_Node_find(last_node, atoi(buff));
-         HTS_get_pattern_token(fp, buff);       /* load question at this node */
-
+         if (node == NULL) {
+            HTS_Tree_clear(tree);
+            return FALSE;
+         }
+         if (HTS_get_pattern_token(fp, buff) == FALSE) {
+            HTS_Tree_clear(tree);
+            return FALSE;
+         }
          node->quest = HTS_Question_find_question(question, buff);
+         if (node->quest == NULL) {
+            HTS_Tree_clear(tree);
+            return FALSE;
+         }
          node->yes = (HTS_Node *) HTS_calloc(1, sizeof(HTS_Node));
          node->no = (HTS_Node *) HTS_calloc(1, sizeof(HTS_Node));
 
-         HTS_get_pattern_token(fp, buff);
+         if (HTS_get_pattern_token(fp, buff) == FALSE) {
+            node->quest = NULL;
+            free(node->yes);
+            free(node->no);
+            HTS_Tree_clear(tree);
+            return FALSE;
+         }
          if (HTS_is_num(buff))
             node->no->index = atoi(buff);
          else
@@ -313,7 +350,13 @@ static void HTS_Tree_load(HTS_Tree * tree, FILE * fp, HTS_Question * question)
          node->no->next = last_node;
          last_node = node->no;
 
-         HTS_get_pattern_token(fp, buff);
+         if (HTS_get_pattern_token(fp, buff) == FALSE) {
+            node->quest = NULL;
+            free(node->yes);
+            free(node->no);
+            HTS_Tree_clear(tree);
+            return FALSE;
+         }
          if (HTS_is_num(buff))
             node->yes->index = atoi(buff);
          else
@@ -324,6 +367,8 @@ static void HTS_Tree_load(HTS_Tree * tree, FILE * fp, HTS_Question * question)
    } else {
       node->pdf = HTS_name2num(buff);
    }
+
+   return TRUE;
 }
 
 /* HTS_Node_search: tree search */
@@ -373,12 +418,19 @@ static void HTS_Window_initialize(HTS_Window * win)
    win->max_width = 0;
 }
 
+static void HTS_Window_clear(HTS_Window * win);
+
 /* HTS_Window_load: load dynamic windows */
-static void HTS_Window_load(HTS_Window * win, FILE ** fp, int size)
+static HTS_Boolean HTS_Window_load(HTS_Window * win, HTS_File ** fp, int size)
 {
    int i, j;
    int fsize, length;
    char buff[HTS_MAXBUFLEN];
+   HTS_Boolean result = TRUE;
+
+   /* check */
+   if (win == NULL || fp == NULL || size <= 0)
+      return FALSE;
 
    win->size = size;
    win->l_width = (int *) HTS_calloc(win->size, sizeof(int));
@@ -386,13 +438,25 @@ static void HTS_Window_load(HTS_Window * win, FILE ** fp, int size)
    win->coefficient = (double **) HTS_calloc(win->size, sizeof(double *));
    /* set delta coefficents */
    for (i = 0; i < win->size; i++) {
-      HTS_get_token(fp[i], buff);
-      fsize = atoi(buff);
+      if (HTS_get_token(fp[i], buff) == FALSE) {
+         result = FALSE;
+         fsize = 1;
+      } else {
+         fsize = atoi(buff);
+         if (fsize <= 0) {
+            result = FALSE;
+            fsize = 1;
+         }
+      }
       /* read coefficients */
       win->coefficient[i] = (double *) HTS_calloc(fsize, sizeof(double));
       for (j = 0; j < fsize; j++) {
-         HTS_get_token(fp[i], buff);
-         win->coefficient[i][j] = (double) atof(buff);
+         if (HTS_get_token(fp[i], buff) == FALSE) {
+            result = FALSE;
+            win->coefficient[i][j] = 0.0;
+         } else {
+            win->coefficient[i][j] = (double) atof(buff);
+         }
       }
       /* set pointer */
       length = fsize / 2;
@@ -410,6 +474,12 @@ static void HTS_Window_load(HTS_Window * win, FILE ** fp, int size)
       if (win->max_width < abs(win->r_width[i]))
          win->max_width = abs(win->r_width[i]);
    }
+
+   if (result == FALSE) {
+      HTS_Window_clear(win);
+      return FALSE;
+   }
+   return TRUE;
 }
 
 /* HTS_Window_clear: free dynamic window */
@@ -443,95 +513,120 @@ static void HTS_Model_initialize(HTS_Model * model)
    model->question = NULL;
 }
 
+static void HTS_Model_clear(HTS_Model * model);
+
 /* HTS_Model_load_pdf: load pdfs */
-static void HTS_Model_load_pdf(HTS_Model * model, FILE * fp, int ntree,
-                               HTS_Boolean msd_flag)
+static HTS_Boolean HTS_Model_load_pdf(HTS_Model * model, HTS_File * fp, int ntree, HTS_Boolean msd_flag)
 {
    int i, j, k, l, m;
    float temp;
    int ssize;
+   HTS_Boolean result = TRUE;
 
    /* check */
-   if (fp == NULL)
+   if (model == NULL || fp == NULL || ntree <= 0) {
       HTS_error(1, "HTS_Model_load_pdf: File for pdfs is not specified.\n");
+      return FALSE;
+   }
 
    /* load pdf */
    model->ntree = ntree;
    /* read MSD flag */
    HTS_fread_big_endian(&i, sizeof(int), 1, fp);
-   if ((i != 0 || msd_flag != FALSE) && (i != 1 || msd_flag != TRUE))
+   if ((i != 0 || msd_flag != FALSE) && (i != 1 || msd_flag != TRUE)) {
       HTS_error(1, "HTS_Model_load_pdf: Failed to load header of pdfs.\n");
+      HTS_Model_initialize(model);
+      return FALSE;
+   }
    /* read stream size */
    HTS_fread_big_endian(&ssize, sizeof(int), 1, fp);
-   if (ssize < 1)
+   if (ssize < 1) {
       HTS_error(1, "HTS_Model_load_pdf: Failed to load header of pdfs.\n");
+      HTS_Model_initialize(model);
+      return FALSE;
+   }
    /* read vector size */
    HTS_fread_big_endian(&model->vector_length, sizeof(int), 1, fp);
-   if (model->vector_length < 0)
-      HTS_error(1,
-                "HTS_Model_load_pdf: # of HMM states %d should be positive.\n",
-                model->vector_length);
+   if (model->vector_length <= 0) {
+      HTS_error(1, "HTS_Model_load_pdf: # of HMM states %d should be positive.\n", model->vector_length);
+      HTS_Model_initialize(model);
+      return FALSE;
+   }
    model->npdf = (int *) HTS_calloc(ntree, sizeof(int));
    model->npdf -= 2;
    /* read the number of pdfs */
-   HTS_fread_big_endian(&model->npdf[2], sizeof(int), ntree, fp);
+   if (HTS_fread_big_endian(&model->npdf[2], sizeof(int), ntree, fp) != ntree)
+      result = FALSE;
    for (i = 2; i <= ntree + 1; i++)
-      if (model->npdf[i] < 0)
-         HTS_error(1,
-                   "HTS_Model_load_pdf: # of pdfs at %d-th state should be positive.\n",
-                   i);
+      if (model->npdf[i] <= 0) {
+         HTS_error(1, "HTS_Model_load_pdf: # of pdfs at %d-th state should be positive.\n", i);
+         result = FALSE;
+         break;
+      }
+   if (result == FALSE) {
+      model->npdf += 2;
+      free(model->npdf);
+      HTS_Model_initialize(model);
+      return FALSE;
+   }
    model->pdf = (double ***) HTS_calloc(ntree, sizeof(double **));
    model->pdf -= 2;
    /* read means and variances */
    if (msd_flag) {              /* for MSD */
       for (j = 2; j <= ntree + 1; j++) {
-         model->pdf[j] = (double **)
-             HTS_calloc(model->npdf[j], sizeof(double *));
+         model->pdf[j] = (double **) HTS_calloc(model->npdf[j], sizeof(double *));
          model->pdf[j]--;
          for (k = 1; k <= model->npdf[j]; k++) {
-            model->pdf[j][k] = (double *)
-                HTS_calloc(2 * model->vector_length + 1, sizeof(double));
+            model->pdf[j][k] = (double *) HTS_calloc(2 * model->vector_length + 1, sizeof(double));
             for (l = 0; l < ssize; l++) {
                for (m = 0; m < model->vector_length / ssize; m++) {
-                  HTS_fread_big_endian(&temp, sizeof(float), 1, fp);
-                  model->pdf[j][k][l * model->vector_length / ssize + m] =
-                      (double) temp;
-                  HTS_fread_big_endian(&temp, sizeof(float), 1, fp);
-                  model->pdf[j][k][l * model->vector_length / ssize + m +
-                                   model->vector_length] = (double) temp;
+                  if (HTS_fread_big_endian(&temp, sizeof(float), 1, fp) != 1)
+                     result = FALSE;
+                  model->pdf[j][k][l * model->vector_length / ssize + m] = (double) temp;
+                  if (HTS_fread_big_endian(&temp, sizeof(float), 1, fp) != 1)
+                     result = FALSE;
+                  model->pdf[j][k][l * model->vector_length / ssize + m + model->vector_length] = (double) temp;
                }
-               HTS_fread_big_endian(&temp, sizeof(float), 1, fp);
+               if (HTS_fread_big_endian(&temp, sizeof(float), 1, fp) != 1)
+                  result = FALSE;
                if (l == 0) {
-                  if (temp < 0.0 || temp > 1.0)
-                     HTS_error(1,
-                               "HTS_Model_load_pdf: MSD weight should be within 0.0 to 1.0.\n");
+                  if (temp < 0.0 || temp > 1.0) {
+                     HTS_error(1, "HTS_Model_load_pdf: MSD weight should be within 0.0 to 1.0.\n");
+                     result = FALSE;
+                  }
                   model->pdf[j][k][2 * model->vector_length] = (double) temp;
                }
-               HTS_fread_big_endian(&temp, sizeof(float), 1, fp);
+               if (HTS_fread_big_endian(&temp, sizeof(float), 1, fp) != 1)
+                  result = FALSE;
             }
          }
       }
    } else {                     /* for non MSD */
       for (j = 2; j <= ntree + 1; j++) {
-         model->pdf[j] =
-             (double **) HTS_calloc(model->npdf[j], sizeof(double *));
+         model->pdf[j] = (double **) HTS_calloc(model->npdf[j], sizeof(double *));
          model->pdf[j]--;
          for (k = 1; k <= model->npdf[j]; k++) {
-            model->pdf[j][k] =
-                (double *) HTS_calloc(2 * model->vector_length, sizeof(double));
+            model->pdf[j][k] = (double *) HTS_calloc(2 * model->vector_length, sizeof(double));
             for (l = 0; l < model->vector_length; l++) {
-               HTS_fread_big_endian(&temp, sizeof(float), 1, fp);
+               if (HTS_fread_big_endian(&temp, sizeof(float), 1, fp) != 1)
+                  result = FALSE;
                model->pdf[j][k][l] = (double) temp;
-               HTS_fread_big_endian(&temp, sizeof(float), 1, fp);
+               if (HTS_fread_big_endian(&temp, sizeof(float), 1, fp) != 1)
+                  result = FALSE;
                model->pdf[j][k][l + model->vector_length] = (double) temp;
             }
          }
       }
    }
+   if (result == FALSE) {
+      HTS_Model_clear(model);
+      return FALSE;
+   }
+   return TRUE;
 }
 
 /* HTS_Model_load_tree: load trees */
-static void HTS_Model_load_tree(HTS_Model * model, FILE * fp)
+static HTS_Boolean HTS_Model_load_tree(HTS_Model * model, HTS_File * fp)
 {
    char buff[HTS_MAXBUFLEN];
    HTS_Question *question, *last_question;
@@ -539,18 +634,24 @@ static void HTS_Model_load_tree(HTS_Model * model, FILE * fp)
    int state;
 
    /* check */
-   if (fp == NULL)
+   if (model == NULL || fp == NULL) {
       HTS_error(1, "HTS_Model_load_tree: File for trees is not specified.\n");
+      return FALSE;
+   }
 
    model->ntree = 0;
    last_question = NULL;
    last_tree = NULL;
-   while (!feof(fp)) {
+   while (!HTS_feof(fp)) {
       HTS_get_pattern_token(fp, buff);
       /* parse questions */
       if (strcmp(buff, "QS") == 0) {
          question = (HTS_Question *) HTS_calloc(1, sizeof(HTS_Question));
-         HTS_Question_load(question, fp);
+         if (HTS_Question_load(question, fp) == FALSE) {
+            free(question);
+            HTS_Model_clear(model);
+            return FALSE;
+         }
          if (model->question)
             last_question->next = question;
          else
@@ -567,7 +668,11 @@ static void HTS_Model_load_tree(HTS_Model * model, FILE * fp)
          tree->head = NULL;
          tree->state = state;
          HTS_Tree_parse_pattern(tree, buff);
-         HTS_Tree_load(tree, fp, model->question);
+         if (HTS_Tree_load(tree, fp, model->question) == FALSE) {
+            free(tree);
+            HTS_Model_clear(model);
+            return FALSE;
+         }
          if (model->tree)
             last_tree->next = tree;
          else
@@ -578,8 +683,11 @@ static void HTS_Model_load_tree(HTS_Model * model, FILE * fp)
       }
    }
    /* No Tree information in tree file */
-   if (model->tree == NULL)
+   if (model->tree == NULL) {
       HTS_error(1, "HTS_Model_load_tree: No trees are loaded.\n");
+      return FALSE;
+   }
+   return TRUE;
 }
 
 /* HTS_Model_clear: free pdfs and trees */
@@ -627,70 +735,93 @@ static void HTS_Stream_initialize(HTS_Stream * stream)
    stream->interpolation_size = 0;
 }
 
+static void HTS_Stream_clear(HTS_Stream * stream);
+
 /* HTS_Stream_load_pdf: load pdf */
-static void HTS_Stream_load_pdf(HTS_Stream * stream, FILE ** fp, int ntree,
-                                HTS_Boolean msd_flag, int interpolation_size)
+static HTS_Boolean HTS_Stream_load_pdf(HTS_Stream * stream, HTS_File ** fp, int ntree, HTS_Boolean msd_flag, int interpolation_size)
 {
    int i;
+   HTS_Boolean result = TRUE;
 
    /* initialize */
    stream->msd_flag = msd_flag;
    stream->interpolation_size = interpolation_size;
-   stream->model =
-       (HTS_Model *) HTS_calloc(interpolation_size, sizeof(HTS_Model));
+   stream->model = (HTS_Model *) HTS_calloc(interpolation_size, sizeof(HTS_Model));
    /* load pdfs */
    for (i = 0; i < stream->interpolation_size; i++) {
       HTS_Model_initialize(&stream->model[i]);
-      HTS_Model_load_pdf(&stream->model[i], fp[i], ntree, stream->msd_flag);
+      if (HTS_Model_load_pdf(&stream->model[i], fp[i], ntree, stream->msd_flag) == FALSE)
+         result = FALSE;
+   }
+   if (result == FALSE) {
+      HTS_Stream_clear(stream);
+      return FALSE;
    }
    /* check */
-   for (i = 1; i < stream->interpolation_size; i++)
-      if (stream->model[0].vector_length != stream->model[1].vector_length)
-         HTS_error(1,
-                   "HTS_Stream_load_pdf: # of states are different in between given modelsets.\n");
+   for (i = 1; i < stream->interpolation_size; i++) {
+      if (stream->model[0].vector_length != stream->model[1].vector_length) {
+         HTS_error(1, "HTS_Stream_load_pdf: # of states are different in between given modelsets.\n");
+         HTS_Stream_clear(stream);
+         return FALSE;
+      }
+   }
    /* set */
    stream->vector_length = stream->model[0].vector_length;
+
+   return TRUE;
 }
 
 /* HTS_Stream_load_pdf_and_tree: load PDFs and trees */
-static void HTS_Stream_load_pdf_and_tree(HTS_Stream * stream, FILE ** pdf_fp,
-                                         FILE ** tree_fp, HTS_Boolean msd_flag,
-                                         int interpolation_size)
+static HTS_Boolean HTS_Stream_load_pdf_and_tree(HTS_Stream * stream, HTS_File ** pdf_fp, HTS_File ** tree_fp, HTS_Boolean msd_flag, int interpolation_size)
 {
    int i;
+   HTS_Boolean result = TRUE;
 
    /* initialize */
    stream->msd_flag = msd_flag;
    stream->interpolation_size = interpolation_size;
-   stream->model =
-       (HTS_Model *) HTS_calloc(interpolation_size, sizeof(HTS_Model));
+   stream->model = (HTS_Model *) HTS_calloc(interpolation_size, sizeof(HTS_Model));
    /* load */
    for (i = 0; i < stream->interpolation_size; i++) {
-      if (!pdf_fp[i])
-         HTS_error(1,
-                   "HTS_Stream_load_pdf_and_tree: File for duration PDFs is not specified.\n");
-      if (!tree_fp[i])
-         HTS_error(1,
-                   "HTS_Stream_load_pdf_and_tree: File for duration trees is not specified.\n");
+      if (!pdf_fp[i]) {
+         HTS_error(1, "HTS_Stream_load_pdf_and_tree: File for duration PDFs is not specified.\n");
+         HTS_Stream_clear(stream);
+         return FALSE;
+      }
+      if (!tree_fp[i]) {
+         HTS_error(1, "HTS_Stream_load_pdf_and_tree: File for duration trees is not specified.\n");
+         HTS_Stream_clear(stream);
+         return FALSE;
+      }
       HTS_Model_initialize(&stream->model[i]);
-      HTS_Model_load_tree(&stream->model[i], tree_fp[i]);
-      HTS_Model_load_pdf(&stream->model[i], pdf_fp[i], stream->model[i].ntree,
-                         stream->msd_flag);
+      result = HTS_Model_load_tree(&stream->model[i], tree_fp[i]);
+      if (result == FALSE) {
+         HTS_Stream_clear(stream);
+         return FALSE;
+      }
+      result = HTS_Model_load_pdf(&stream->model[i], pdf_fp[i], stream->model[i].ntree, stream->msd_flag);
+      if (result == FALSE) {
+         HTS_Stream_clear(stream);
+         return FALSE;
+      }
    }
    /* check */
    for (i = 1; i < stream->interpolation_size; i++)
-      if (stream->model[0].vector_length != stream->model[i].vector_length)
-         HTS_error(1,
-                   "HTS_Stream_load_pdf_and_tree: Vector sizes of state output vectors are different in between given modelsets.\n");
+      if (stream->model[0].vector_length != stream->model[i].vector_length) {
+         HTS_error(1, "HTS_Stream_load_pdf_and_tree: Vector sizes of state output vectors are different in between given modelsets.\n");
+         HTS_Stream_clear(stream);
+         return FALSE;
+      }
    /* set */
    stream->vector_length = stream->model[0].vector_length;
+
+   return TRUE;
 }
 
 /* HTS_Stream_load_dynamic_window: load windows */
-static void HTS_Stream_load_dynamic_window(HTS_Stream * stream, FILE ** fp,
-                                           int size)
+static HTS_Boolean HTS_Stream_load_dynamic_window(HTS_Stream * stream, HTS_File ** fp, int size)
 {
-   HTS_Window_load(&stream->window, fp, size);
+   return HTS_Window_load(&stream->window, fp, size);
 }
 
 /* HTS_Stream_clear: free stream */
@@ -719,40 +850,64 @@ void HTS_ModelSet_initialize(HTS_ModelSet * ms, int nstream)
 }
 
 /* HTS_ModelSet_load_duration: load duration model and number of state */
-void HTS_ModelSet_load_duration(HTS_ModelSet * ms, FILE ** pdf_fp,
-                                FILE ** tree_fp, int interpolation_size)
+HTS_Boolean HTS_ModelSet_load_duration(HTS_ModelSet * ms, HTS_File ** pdf_fp, HTS_File ** tree_fp, int interpolation_size)
 {
    /* check */
-   if (pdf_fp == NULL)
-      HTS_error(1,
-                "HTS_ModelSet_load_duration: File for duration PDFs is not specified.\n");
-   if (tree_fp == NULL)
-      HTS_error(1,
-                "HTS_ModelSet_load_duration: File for duration trees is not specified.\n");
+   if (ms == NULL) {
+      return FALSE;
+   }
+   if (interpolation_size <= 0) {
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
+   if (pdf_fp == NULL) {
+      HTS_error(1, "HTS_ModelSet_load_duration: File for duration PDFs is not specified.\n");
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
+   if (tree_fp == NULL) {
+      HTS_error(1, "HTS_ModelSet_load_duration: File for duration trees is not specified.\n");
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
 
-   HTS_Stream_load_pdf_and_tree(&ms->duration, pdf_fp, tree_fp, FALSE,
-                                interpolation_size);
+   if (HTS_Stream_load_pdf_and_tree(&ms->duration, pdf_fp, tree_fp, FALSE, interpolation_size) == FALSE) {
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
    ms->nstate = ms->duration.vector_length;
+
+   return TRUE;
 }
 
 /* HTS_ModelSet_load_parameter: load model */
-void HTS_ModelSet_load_parameter(HTS_ModelSet * ms, FILE ** pdf_fp,
-                                 FILE ** tree_fp, FILE ** win_fp,
-                                 int stream_index, HTS_Boolean msd_flag,
-                                 int window_size, int interpolation_size)
+HTS_Boolean HTS_ModelSet_load_parameter(HTS_ModelSet * ms, HTS_File ** pdf_fp, HTS_File ** tree_fp, HTS_File ** win_fp, int stream_index, HTS_Boolean msd_flag, int window_size, int interpolation_size)
 {
    int i;
 
    /* check */
-   if (pdf_fp == NULL)
-      HTS_error(1,
-                "HTS_ModelSet_load_parameter: File for pdfs is not specified.\n");
-   if (tree_fp == NULL)
-      HTS_error(1,
-                "HTS_ModelSet_load_parameter: File for wins is not specified.\n");
-   if (win_fp == NULL)
-      HTS_error(1,
-                "HTS_ModelSet_load_parameter: File for wins is not specified.\n");
+   if (ms == NULL) {
+      return FALSE;
+   }
+   if (stream_index < 0 || stream_index >= ms->nstream || window_size <= 0 || interpolation_size <= 0) {
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
+   if (pdf_fp == NULL) {
+      HTS_error(1, "HTS_ModelSet_load_parameter: File for pdfs is not specified.\n");
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
+   if (tree_fp == NULL) {
+      HTS_error(1, "HTS_ModelSet_load_parameter: File for wins is not specified.\n");
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
+   if (win_fp == NULL) {
+      HTS_error(1, "HTS_ModelSet_load_parameter: File for wins is not specified.\n");
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
    /* initialize */
    if (!ms->stream) {
       ms->stream = (HTS_Stream *) HTS_calloc(ms->nstream, sizeof(HTS_Stream));
@@ -760,34 +915,55 @@ void HTS_ModelSet_load_parameter(HTS_ModelSet * ms, FILE ** pdf_fp,
          HTS_Stream_initialize(&ms->stream[i]);
    }
    /* load */
-   HTS_Stream_load_pdf_and_tree(&ms->stream[stream_index], pdf_fp, tree_fp,
-                                msd_flag, interpolation_size);
-   HTS_Stream_load_dynamic_window(&ms->stream[stream_index], win_fp,
-                                  window_size);
+   if (HTS_Stream_load_pdf_and_tree(&ms->stream[stream_index], pdf_fp, tree_fp, msd_flag, interpolation_size) == FALSE) {
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
+   if (HTS_Stream_load_dynamic_window(&ms->stream[stream_index], win_fp, window_size) == FALSE) {
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
+
+   return TRUE;
 }
 
 /* HTS_ModelSet_load_gv: load GV model */
-void HTS_ModelSet_load_gv(HTS_ModelSet * ms, FILE ** pdf_fp, FILE ** tree_fp,
-                          int stream_index, int interpolation_size)
+HTS_Boolean HTS_ModelSet_load_gv(HTS_ModelSet * ms, HTS_File ** pdf_fp, HTS_File ** tree_fp, int stream_index, int interpolation_size)
 {
    int i;
 
    /* check */
-   if (pdf_fp == NULL)
-      HTS_error(1,
-                "HTS_ModelSet_load_gv: File for GV pdfs is not specified.\n");
+   if (ms == NULL) {
+      return FALSE;
+   }
+   if (stream_index < 0 || stream_index >= ms->nstream || interpolation_size <= 0) {
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
+   if (pdf_fp == NULL) {
+      HTS_error(1, "HTS_ModelSet_load_gv: File for GV pdfs is not specified.\n");
+      HTS_ModelSet_clear(ms);
+      return FALSE;
+   }
    /* initialize */
    if (!ms->gv) {
       ms->gv = (HTS_Stream *) HTS_calloc(ms->nstream, sizeof(HTS_Stream));
       for (i = 0; i < ms->nstream; i++)
          HTS_Stream_initialize(&ms->gv[i]);
    }
-   if (tree_fp)
-      HTS_Stream_load_pdf_and_tree(&ms->gv[stream_index], pdf_fp, tree_fp,
-                                   FALSE, interpolation_size);
-   else
-      HTS_Stream_load_pdf(&ms->gv[stream_index], pdf_fp, 1, FALSE,
-                          interpolation_size);
+   if (tree_fp) {
+      if (HTS_Stream_load_pdf_and_tree(&ms->gv[stream_index], pdf_fp, tree_fp, FALSE, interpolation_size) == FALSE) {
+         HTS_ModelSet_clear(ms);
+         return FALSE;
+      }
+   } else {
+      if (HTS_Stream_load_pdf(&ms->gv[stream_index], pdf_fp, 1, FALSE, interpolation_size) == FALSE) {
+         HTS_ModelSet_clear(ms);
+         return FALSE;
+      }
+   }
+
+   return TRUE;
 }
 
 /* HTS_ModelSet_have_gv_tree: if context-dependent GV is used, return true */
@@ -802,10 +978,11 @@ HTS_Boolean HTS_ModelSet_have_gv_tree(HTS_ModelSet * ms, int stream_index)
 }
 
 /* HTS_ModelSet_load_gv_switch: load GV switch */
-void HTS_ModelSet_load_gv_switch(HTS_ModelSet * ms, FILE * fp)
+HTS_Boolean HTS_ModelSet_load_gv_switch(HTS_ModelSet * ms, HTS_File * fp)
 {
    if (fp != NULL)
-      HTS_Model_load_tree(&ms->gv_switch, fp);
+      return HTS_Model_load_tree(&ms->gv_switch, fp);
+   return FALSE;
 }
 
 /* HTS_ModelSet_have_gv_switch: if GV switch is used, return true */
@@ -848,26 +1025,21 @@ int HTS_ModelSet_get_window_size(HTS_ModelSet * ms, int stream_index)
 }
 
 /* HTS_ModelSet_get_window_left_width: get left width of dynamic window */
-int HTS_ModelSet_get_window_left_width(HTS_ModelSet * ms, int stream_index,
-                                       int window_index)
+int HTS_ModelSet_get_window_left_width(HTS_ModelSet * ms, int stream_index, int window_index)
 {
    return ms->stream[stream_index].window.l_width[window_index];
 }
 
 /* HTS_ModelSet_get_window_right_width: get right width of dynamic window */
-int HTS_ModelSet_get_window_right_width(HTS_ModelSet * ms, int stream_index,
-                                        int window_index)
+int HTS_ModelSet_get_window_right_width(HTS_ModelSet * ms, int stream_index, int window_index)
 {
    return ms->stream[stream_index].window.r_width[window_index];
 }
 
 /* HTS_ModelSet_get_window_coefficient: get coefficient of dynamic window */
-double HTS_ModelSet_get_window_coefficient(HTS_ModelSet * ms, int stream_index,
-                                           int window_index,
-                                           int coefficient_index)
+double HTS_ModelSet_get_window_coefficient(HTS_ModelSet * ms, int stream_index, int window_index, int coefficient_index)
 {
-   return ms->stream[stream_index].window.
-       coefficient[window_index][coefficient_index];
+   return ms->stream[stream_index].window.coefficient[window_index][coefficient_index];
 }
 
 /* HTS_ModelSet_get_window_max_width: get max width of dynamic window */
@@ -883,8 +1055,7 @@ int HTS_ModelSet_get_duration_interpolation_size(HTS_ModelSet * ms)
 }
 
 /* HTS_ModelSet_get_parameter_interpolation_size: get interpolation size (parameter model) */
-int HTS_ModelSet_get_parameter_interpolation_size(HTS_ModelSet * ms,
-                                                  int stream_index)
+int HTS_ModelSet_get_parameter_interpolation_size(HTS_ModelSet * ms, int stream_index)
 {
    return ms->stream[stream_index].interpolation_size;
 }
@@ -906,9 +1077,7 @@ HTS_Boolean HTS_ModelSet_use_gv(HTS_ModelSet * ms, int stream_index)
 }
 
 /* HTS_ModelSet_get_duration_index: get index of duration tree and PDF */
-void HTS_ModelSet_get_duration_index(HTS_ModelSet * ms, char *string,
-                                     int *tree_index, int *pdf_index,
-                                     int interpolation_index)
+void HTS_ModelSet_get_duration_index(HTS_ModelSet * ms, char *string, int *tree_index, int *pdf_index, int interpolation_index)
 {
    HTS_Tree *tree;
    HTS_Pattern *pattern;
@@ -917,8 +1086,7 @@ void HTS_ModelSet_get_duration_index(HTS_ModelSet * ms, char *string,
    find = FALSE;
    (*tree_index) = 2;
    (*pdf_index) = 1;
-   for (tree = ms->duration.model[interpolation_index].tree; tree;
-        tree = tree->next) {
+   for (tree = ms->duration.model[interpolation_index].tree; tree; tree = tree->next) {
       pattern = tree->head;
       if (!pattern)
          find = TRUE;
@@ -932,15 +1100,15 @@ void HTS_ModelSet_get_duration_index(HTS_ModelSet * ms, char *string,
       (*tree_index)++;
    }
 
-   if (tree == NULL)
-      HTS_error(1, "HTS_ModelSet_get_duration_index: Cannot find model %s.\n",
-                string);
+   if (tree == NULL) {
+      HTS_error(1, "HTS_ModelSet_get_duration_index: Cannot find model %s.\n", string);
+      return;
+   }
    (*pdf_index) = HTS_Tree_search_node(tree, string);
 }
 
 /* HTS_ModelSet_get_duration: get duration using interpolation weight */
-void HTS_ModelSet_get_duration(HTS_ModelSet * ms, char *string, double *mean,
-                               double *vari, double *iw)
+void HTS_ModelSet_get_duration(HTS_ModelSet * ms, char *string, double *mean, double *vari, double *iw)
 {
    int i, j;
    int tree_index, pdf_index;
@@ -954,17 +1122,13 @@ void HTS_ModelSet_get_duration(HTS_ModelSet * ms, char *string, double *mean,
       HTS_ModelSet_get_duration_index(ms, string, &tree_index, &pdf_index, i);
       for (j = 0; j < ms->nstate; j++) {
          mean[j] += iw[i] * ms->duration.model[i].pdf[tree_index][pdf_index][j];
-         vari[j] += iw[i] * iw[i] * ms->duration.model[i]
-             .pdf[tree_index][pdf_index][j + vector_length];
+         vari[j] += iw[i] * iw[i] * ms->duration.model[i].pdf[tree_index][pdf_index][j + vector_length];
       }
    }
 }
 
 /* HTS_ModelSet_get_parameter_index: get index of parameter tree and PDF */
-void HTS_ModelSet_get_parameter_index(HTS_ModelSet * ms, char *string,
-                                      int *tree_index, int *pdf_index,
-                                      int stream_index, int state_index,
-                                      int interpolation_index)
+void HTS_ModelSet_get_parameter_index(HTS_ModelSet * ms, char *string, int *tree_index, int *pdf_index, int stream_index, int state_index, int interpolation_index)
 {
    HTS_Tree *tree;
    HTS_Pattern *pattern;
@@ -973,8 +1137,7 @@ void HTS_ModelSet_get_parameter_index(HTS_ModelSet * ms, char *string,
    find = FALSE;
    (*tree_index) = 2;
    (*pdf_index) = 1;
-   for (tree = ms->stream[stream_index].model[interpolation_index].tree; tree;
-        tree = tree->next) {
+   for (tree = ms->stream[stream_index].model[interpolation_index].tree; tree; tree = tree->next) {
       if (tree->state == state_index) {
          pattern = tree->head;
          if (!pattern)
@@ -990,16 +1153,15 @@ void HTS_ModelSet_get_parameter_index(HTS_ModelSet * ms, char *string,
       (*tree_index)++;
    }
 
-   if (tree == NULL)
-      HTS_error(1, "HTS_ModelSet_get_parameter_index: Cannot find model %s.\n",
-                string);
+   if (tree == NULL) {
+      HTS_error(1, "HTS_ModelSet_get_parameter_index: Cannot find model %s.\n", string);
+      return;
+   }
    (*pdf_index) = HTS_Tree_search_node(tree, string);
 }
 
 /* HTS_ModelSet_get_parameter: get parameter using interpolation weight */
-void HTS_ModelSet_get_parameter(HTS_ModelSet * ms, char *string, double *mean,
-                                double *vari, double *msd, int stream_index,
-                                int state_index, double *iw)
+void HTS_ModelSet_get_parameter(HTS_ModelSet * ms, char *string, double *mean, double *vari, double *msd, int stream_index, int state_index, double *iw)
 {
    int i, j;
    int tree_index, pdf_index;
@@ -1012,11 +1174,9 @@ void HTS_ModelSet_get_parameter(HTS_ModelSet * ms, char *string, double *mean,
    if (msd)
       *msd = 0.0;
    for (i = 0; i < ms->stream[stream_index].interpolation_size; i++) {
-      HTS_ModelSet_get_parameter_index(ms, string, &tree_index, &pdf_index,
-                                       stream_index, state_index, i);
+      HTS_ModelSet_get_parameter_index(ms, string, &tree_index, &pdf_index, stream_index, state_index, i);
       for (j = 0; j < vector_length; j++) {
-         mean[j] += iw[i] *
-             ms->stream[stream_index].model[i].pdf[tree_index][pdf_index][j];
+         mean[j] += iw[i] * ms->stream[stream_index].model[i].pdf[tree_index][pdf_index][j];
          vari[j] += iw[i] * iw[i] * ms->stream[stream_index].model[i]
              .pdf[tree_index][pdf_index][j + vector_length];
       }
@@ -1028,9 +1188,7 @@ void HTS_ModelSet_get_parameter(HTS_ModelSet * ms, char *string, double *mean,
 }
 
 /* HTS_ModelSet_get_gv_index: get index of GV tree and PDF */
-void HTS_ModelSet_get_gv_index(HTS_ModelSet * ms, char *string, int *tree_index,
-                               int *pdf_index, int stream_index,
-                               int interpolation_index)
+void HTS_ModelSet_get_gv_index(HTS_ModelSet * ms, char *string, int *tree_index, int *pdf_index, int stream_index, int interpolation_index)
 {
    HTS_Tree *tree;
    HTS_Pattern *pattern;
@@ -1042,8 +1200,7 @@ void HTS_ModelSet_get_gv_index(HTS_ModelSet * ms, char *string, int *tree_index,
 
    if (HTS_ModelSet_have_gv_tree(ms, stream_index) == FALSE)
       return;
-   for (tree = ms->gv[stream_index].model[interpolation_index].tree; tree;
-        tree = tree->next) {
+   for (tree = ms->gv[stream_index].model[interpolation_index].tree; tree; tree = tree->next) {
       pattern = tree->head;
       if (!pattern)
          find = TRUE;
@@ -1057,15 +1214,15 @@ void HTS_ModelSet_get_gv_index(HTS_ModelSet * ms, char *string, int *tree_index,
       (*tree_index)++;
    }
 
-   if (tree == NULL)
-      HTS_error(1, "HTS_ModelSet_get_gv_index: Cannot find model %s.\n",
-                string);
+   if (tree == NULL) {
+      HTS_error(1, "HTS_ModelSet_get_gv_index: Cannot find model %s.\n", string);
+      return;
+   }
    (*pdf_index) = HTS_Tree_search_node(tree, string);
 }
 
 /* HTS_ModelSet_get_gv: get GV using interpolation weight */
-void HTS_ModelSet_get_gv(HTS_ModelSet * ms, char *string, double *mean,
-                         double *vari, int stream_index, double *iw)
+void HTS_ModelSet_get_gv(HTS_ModelSet * ms, char *string, double *mean, double *vari, int stream_index, double *iw)
 {
    int i, j;
    int tree_index, pdf_index;
@@ -1076,11 +1233,9 @@ void HTS_ModelSet_get_gv(HTS_ModelSet * ms, char *string, double *mean,
       vari[i] = 0.0;
    }
    for (i = 0; i < ms->gv[stream_index].interpolation_size; i++) {
-      HTS_ModelSet_get_gv_index(ms, string, &tree_index, &pdf_index,
-                                stream_index, i);
+      HTS_ModelSet_get_gv_index(ms, string, &tree_index, &pdf_index, stream_index, i);
       for (j = 0; j < vector_length; j++) {
-         mean[j] += iw[i] *
-             ms->gv[stream_index].model[i].pdf[tree_index][pdf_index][j];
+         mean[j] += iw[i] * ms->gv[stream_index].model[i].pdf[tree_index][pdf_index][j];
          vari[j] += iw[i] * iw[i] * ms->gv[stream_index].model[i]
              .pdf[tree_index][pdf_index][j + vector_length];
       }
@@ -1088,8 +1243,7 @@ void HTS_ModelSet_get_gv(HTS_ModelSet * ms, char *string, double *mean,
 }
 
 /* HTS_ModelSet_get_gv_switch_index: get index of GV switch tree and PDF */
-void HTS_ModelSet_get_gv_switch_index(HTS_ModelSet * ms, char *string,
-                                      int *tree_index, int *pdf_index)
+void HTS_ModelSet_get_gv_switch_index(HTS_ModelSet * ms, char *string, int *tree_index, int *pdf_index)
 {
    HTS_Tree *tree;
    HTS_Pattern *pattern;
@@ -1112,9 +1266,10 @@ void HTS_ModelSet_get_gv_switch_index(HTS_ModelSet * ms, char *string,
       (*tree_index)++;
    }
 
-   if (tree == NULL)
-      HTS_error(1, "HTS_ModelSet_get_gv_switch_index: Cannot find model %s.\n",
-                string);
+   if (tree == NULL) {
+      HTS_error(1, "HTS_ModelSet_get_gv_switch_index: Cannot find model %s.\n", string);
+      return;
+   }
    (*pdf_index) = HTS_Tree_search_node(tree, string);
 }
 
